occMod_2SP<-function(psi=call(),psi.cov=data$unitcov,
                         p=call(),p.cov=cbind(data$unitcov,data$survcov),
                         gamma=call(),gamma.cov=cbind(data$unitcov,data$survcov),
                         epsilon=call(),epsilon.cov=cbind(data$unitcov,data$survcov),
                         parameterization="psiBA", sp.contr=TRUE, modname=NULL, outfile,
                  model=3000,fixed=NULL,initvals=NULL,data,miscopts=miscopts){

#' Fit two species, static occupancy (single season), or dynamic occupancy (multi-season) model
#'
#' This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="so.2sp.1"} or \code{type="so.2sp.2"} depending on the parameterisation required. NOTE THAT THERE MAY BE SOME CHANGES TO HOW THIS MODEL IS IMPLEMENTED IN THE NEAR FUTURE.
#'
#'  Note:  This function assumes data are in "compressed" format (ie.,
#'  \itemize{
#'        \item 0=neither species detected,
#'        \item 1=only species A detected,
#'        \item 2=only species B detected,
#'        \item 3=both species detected)
#'  }
#'        "Stacked" format can easily be converted to "compressed" format by:
#'     \preformatted{
#'     nsites=nrow(det.data)/2
#'     new.det.data=det.data[1:nsites,]+2*det.data[nsites+1:nsites,]
#'     }
#'
#' @param psi The right-hand side of the formula for the model to fit for occupancy probability.
#' The terms \code{SP} and \code{INT} can be used to define a species effect on occupancy and an
#' occurrence-level interaction between species accordingly, without them being defined in \code{psi.cov}.
#' \itemize{
#' \item{\code{SP}}{ species effect on occupancy (psiBA, psiBa != psiA)}
#' \item{\code{INT}}{ interaction effect on occupancy of species A (psiBa != psiBA)}
#' }
#' @param psi.cov A data frame containing the unit-specific covariates to use for the occupancy component
#' of the model.
#' @param p The right-hand side of the formula for the model to fit for detection probability. The terms
#' \code{SP}, \code{INT_o} and \code{INT_d} can be used to define a species effect on detection, a
#' detection-level interaction where the occurrence of one species changes the detection probability of
#' the other species and a second detection-level interaction where the detection of one species changes
#' the detection probability of the other species in the same survey. These terms do not have to be
#' defined as variables in \code{p.cov}.
#' \itemize{
#' \item{\code{SP}}{ species effect on detection (pB,rB? != pA,rA)}
#' \item{\code{INT_o}}{ interaction effect of occupancy on detection (p? != r?)}
#' \item{\code{INT_d}}{ interaction effect of detection on detection (rB? != rA)}
#' \item{\code{SP:INT_o}}{ interaction effect of occupancy and detection on detection (rBa != rBA)}
#' \item{\code{ID}}{ all 5 parameters (pA,pB,rA,rBA,rBa) different (same as ~SP+INT_o+INT_d+SP:INT_o)}
#'
#' }
#' @param p.cov A data frame containing the survey-specific covariates to use for the detection component
#' of the model.
#' @param gamma The right-hand side of the formula for the model to fit for colonization probability. The
#' following terms can be used to define effects on colonization:
#' \itemize{
#' \item{\code{SP}}{ species effect on colonization (gammaB != gammaA)}
#' \item{\code{SEASON}}{ seasonal effect on colonization (gammaXX(t))}
#' \item{\code{INT_Ba}}{ interaction of effect on colonization for species B when species A was, or was
#' not present (gammaBA != gammaBa)}
#' \item{\code{INT_B_a}}{ interaction of effect on colonization for species B when species A was present
#' in the previous season and was, or was not present in succeeding season (gammaB?A != gammaB?a)}
#' \item{\code{INT_Baa}}{ interaction of effect on colonization for species B when species A was not present
#' in the previous season and was, or was not present in succeeding season (gammaBaA != gammaBaa)}
#' \item{\code{INT_Ab}}{ interaction of effect on colonization for species A when species B was, or was
#' not present in the previous season (gammaAB != gammaAb)}
#' }
#' These terms do not have to be
#' defined as variables in \code{p.cov}. For single-season model, set gamma to NULL.
#' @param gamma.cov A data frame containing the site-season-specific covariates to use for the colonization
#' component of the model.  This should be a NxK data frame, where N=number of sites, and
#' K=number of sites*number of seasons.
#' @param epsilon The right-hand side of the formula for the model to fit for extinction probability.
#' The terms \code{SP}, \code{INT_Ba}, \code{INT_B_a}, \code{INT_Baa}, \code{INT_Ab}, \code{SEASON}
#' can be used to define effects on extinction, similar to those defined for colonization.
#' These terms do not have to be defined as variables in \code{p.cov}.
#' @param epsilon.cov A data frame containing the site-specific covariates to use for the extinction
#' component of the model, similar to gam.cov.
#' @param parameterization The parameterization to be used, either "\code{psiBA}" (default) or "\code{phi}" or
#' "\code{nu}" or "\code{alfa}".  The other parameterizations allow the interaction parameter to
#' be included in the model as a true parameter (not derived).
#' @param sp.contr \code{TRUE} (default) or \code{FALSE}. Specifies the type of contrast used for the
#' \code{SP} term. It it used as the \code{contrasts} input in the function \code{\link{contr.treatment}}.
#' @param modname (optional) a string containing the model name
#' @param model The PRESENCE model code. DO NOT CHANGE.
#' @param fixed A single-column matrix containing values for real parameters to be fixed at.
#' \code{rownnames(fixed)} should contain the index of the real parameters to be fixed.
#' @param initvals Initial values for the beta parameters at which PRESENCE begins the optimisation.
#' The default values in PRESENCE is 0.
#' @param data The \code{pao} data object containing the detection data and other information.
#' @param outfile name for output file (use outfile='modname') for outfile named via model name
#' @param miscopts char vector options to send to Presence (eg., 'maxfn=32000'); last element must be ''.
#'
#' @section Parameterizations:
#' \enumerate{
#'   \item \code{psiA, psiB, phi} - original parameterization where phi is the interaction factor, defined
#'   as phi = psiAB / (psiA*psiB)
#'   \item \code{psiA, psiBA, psiBa} - conditional parameterization described in Richmond et.al (2010) where
#'psiA is the unconditional probability of occupancy for species A, psiBA is the conditional probability
#'of occupancy of species B, given species A is present, and psiBa is the conditional probability of
#'occupancy of species B, given species A is not present.
#' \item \code{psiA, psiBa, nu} - Parameterization described in MacKenzie et.al. (2018) where nu is the odds-ration
#'species interaction factor.
#' \item \code{psi-alfaAb, psi-alfaaB, psi-alfaAB} - parameterization described in MacKenzie et.al. (2021) where:
#'    \deqn{logit(\psi^{Ab}) = \alpha_A}
#'    \deqn{logit(\psiAB) = \alphaA + \alphaAB}
#'    \deqn{logit(\psiaB) = \alphaB}
#'    \deqn{logit(\psiAB) = \alphaB + \alphaAB}
#' }
#'
#' @return returns a list of class \code{"occMod"} and \code{"do.1"}.
#'
#' \code{occMod$beta} contains the objects:
#'  \item{psi}{estimated logistic regression coefficients and standard errors for probability of
#'  occurrence in the first year.}
#'  \item{psi.VC}{variance-covariance matrix for logistic regression coefficients for probability
#'  of occurrence in the first year.}
#'  \item{p}{estimated logistic regression coefficients and standard errors for probability of detection.}
#'  \item{p.VC}{variance-covariance matrix for logistic regression coefficients for probability of detection.}
#'  \item{gamma}{estimated logistic regression coefficients and standard errors for probability of colonization.}
#'  \item{gamma.VC}{var-cov matrix for logistic regression coefficients for probability of colonization.}
#'  \item{epsilon}{estimated logistic regression coefficients and standard errors for probability of extinction.}
#'  \item{epsilon.VC}{var-cov matrix for logistic regression coefficients for probability of extinction.}
#'  \item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}
#'
#'  \code{occMod$real} contains the objects:
#'  \item{psi}{estimated probabilities of occurrence for each sampling unit, along with standard errors
#'  and limits of 95\% confidence interval. Estimates are provided for the first season (calculated
#'  directly from the estimated \code{beta} parameters). Estimates for later seasons are provided as
#'  derived parameters. The season for which an estimate applies to can be identified from the final
#'  number of the row names (\code{rownames(psi)}).}
#'  \item{p}{estimated probabilities of detection for each survey, along with standard errors and limits
#'  of 95\% confidence interval.}
#'  \item{gamma}{estimated probabilities of colonization for each sampling unit and season, along with standard errors and limits
#'  of 95\% confidence interval.}
#'  \item{epsilon}{estimated probabilities of extinction for each sampling unit and season, along with standard errors and limits
#'  of 95\% confidence interval.}
#'
#'  \code{occMod$derived} contains the object:
#'  \item{psi}{estimated probabilities of occurrence for each sampling unit for second season onwards,
#'  along with standard errors and limits of 95\% confidence interval. The season for which an estimate
#'  applies to can be identified from the final number of the row names (\code{rownames(psi)}).}
#' @author Darryl MacKenzie and Jim Hines
#' @seealso \code{\link{occMod}}
#' @examples
#'
#' # load a csv data file
#' filename<-system.file("extdata/twosp_exmpl.csv",package="RPresence")
#' csv<-read.csv(filename,as.is=T,header=F)
#' ##
#' nsites=nrow(csv); nsrvys=ncol(csv)  #  set number of sites,surveys from det. data
#' dethist=matrix(as.integer(unlist(csv)),nrow=nsites) # replace missing values(-) with NA
#' cov1=cov2=NULL
#' ##          create input "pao" object, for use with occMod function
#' data=createPao(dethist,unitcov=cov1,survcov=cov2,title="twosp example")
#'
#' ## fit some models...
#' ##  mod1:
#' ##    occupancy: species-specific, no interaction, parameters: psiA, psiBA=psiBa
#' ##    detection: species-specific, p=r
#' mod1<-occMod(model=list(psi~SP,p~SP),data=data,type="so.2sp.1",parameterization="PsiBA")
#'
#' ##  mod2:
#' ##    occupancy: species-specific, interaction, parameters: psiA, psiBA, psiBa
#' ##    detection: species-specific, p=ID (ID means identity matrix = all params different)
#' mod2<-occMod(model=list(psi~SP+INT,p~ID),data=data,type="so.2sp.1",parameterization="phi")
#'
#' ##    create function to print all real and derived estimates from a model...
#' pr <- function(m) {
#'   x=NULL;
#'   for (i in 1:length(m$real))       # for each real parameter
#'     x=rbind(x,unique(m$real[[i]]))  #    get unique rows of estimates
#'   rownames(x)=names(m$real);        # get param name
#'   print(x)                          # print real param estimates
#'   x=NULL;
#'   for (i in 1:length(m$derived))    #  do the same for derived estimates
#'     x=rbind(x,unique(m$derived[[i]]))
#'   rownames(x)=names(m$derived); print(x)
#' }
#' ##  print real and derived estimates for mod2
#' pr(mod2)
#'
#' ## mod3:
#' ##    occupancy: species-specific, interaction,
#' ##    detection: species=specific, interaction p,r (INT_o) and rBA,rBA (INT_d)
#' ##     Note: this is default 2 sp. model (psiA,psiBA,psiBa,pA,pB,rA,rBA,rBa)
#' mod3<-occMod(model=list(psi~SP+INT,p~SP+INT_o+INT_d+SP:INT_o),
#'              data=data,type="so.2sp.1",parameterization="PsiBA")
#' pr(mod3)
#'
#' ## mod4:
#' ##  New Mackenzie 2021 parameterization...
#' #      logit(psiAb)=alphaA
#' #      logit(psiAB)=alphaA+alphaAB
#' #      logit(psiaB)=alphaaB
#' #      logit(psiAB)=alphaaB+alphaAB
#' mod4<- occMod(model=list(psi~-1+ID, p~-1+ID), data=data, type="so.2sp.a")
#' pr(mod4)
#'
#' \dontrun{
#' m1<-occMod(model=list(psi~SP,p~SP,data=data,type="so.2sp.1",parameterization="PsiBA")
#'
#' ## fit some multi-season, 2-species models (using different data)
#' mod0<-occMod(model=list(
#'    psi=psi~SP+INT,               #  PsiA,psiB,psiBa
#'    gamma =gamma~SP+INT_Ab+INT_Ba+INT_B_a+INT_Baa,    # gamAB,gamAb,gamBAA,gamBAa,gamBaA,gamBaa
#'    epsilon=epsilon~SP+INT_Ab+INT_Ba+INT_B_a+INT_Baa, # epsAB,epsAb,epsBAA,epsBAa,epsBaA,epsBaa
#'    p      =p~SP+INT_o+INT_d+SP:INT_o),               #  pA,pB,rA,rBA,rBa
#'    data=data,type="do.2sp.1")
#' mod1<-occMod(model=list(psi=psi~SP+INT,              #  PsiA,psiB,psiBa
#'    gamma =gamma~SP+INT_Ab+INT_Ba,                    #  gamAB,gamAb,gamBA.,gamBa.
#'    epsilon=epsilon~SP+INT_Ab+INT_Ba,                 #  epsAB,epsAb,epsBA.,epsBa.
#'    p      =p~SP+INT_o+INT_d+SP:INT_o),               #  pA,pB,rA,rBA,rBa
#'               data=data,type="do.2sp.1")
#' mod2<-occMod(model=list(psi=psi~SP+INT,                                   #  PsiA,psiB,psiBa
#'                         gamma =gamma~SP,                                  #  gamA.,gamB..
#'                         epsilon=epsilon~SP,                               #  epsA.,epsB..
#'                         p      =p~SP+INT_o+INT_d+SP:INT_o),               #  pA,pB,rA,rBA,rBa
#'               data=data,type="do.2sp.1")
#'
#' tbl=create_aic_table(list(mod0,mod1,mod2))
#' print(tbl$table)
#' }

  if(!(parameterization %in% c("psiBA","phi","nu","alfa"))){
    stop("Invalid parameterisation. Value of parameterization must be either 'psiBA', 'phi', 'nu' or 'alfa'!")
  }
  multiseason=(!is.null(gamma))
  #modname<-paste0(modname,",",parameterization);  #  append parameterization (psiBA or nu) to modelname
  options("na.action"="na.pass")
  N=data$nunits
  if (sum(data$det.data>1,na.rm=T)==0) {   #  if no 2's or 3's, convert "stacked" data to "compressed"
    N=N/2; N=N;
    data$unitnames=gsub("_[AB]","",data$unitnames[1:N])
    ###  compress 2 sets of sites into 1...
    n=nrow(data$det.data)/2
    det1=data$det.data[1:n,]
    det2=data$det.data[n+1:n,]
    ii=(!is.na(det1) & !is.na(det2)); det1[ii]=det1[ii]+2*det2[ii]  # = spA + spB*2 = 0, 1, 2 or 3
    ii=(!is.na(det1) & is.na(det2)); det1[ii]=4+det1[ii]            # spB missing, det=4(spA=0) or 5(spA=1)
    ii=(is.na(det1) & !is.na(det2)); det1[ii]=6+det2[ii]            # spA missing, det=6(spB=0) or 7(spB=1)
    data$det.data=det1
    data$unitcov=data$unitcov[1:N,]
    i=nrow(data$survcov); data$survcov=data$survcov[1:(i/2),]
  }

  SP<-as.factor(c(rep("A",N),rep("B",2*N)))
  contrasts(SP)<-contr.treatment(2,contrasts=sp.contr)

  if (is.null(psi.cov)) psi.cov=data.frame(SP)
  psi.cov$INT<-as.factor(c(rep(1,2*N),rep(2,N)))  #  INT - psiBA,psiBa interaction
  psi.cov$ID<-as.factor(rep(c('A','BA','Ba'),each=N))
  psiform <- as.character(psi); psi.covnames=names(psi.cov); unitcovnames=names(data$unitcov)
  if (data$nunitcov>0)
    for (i in 1:data$nunitcov) {
      if (length(grep(unitcovnames[i],psiform))>0) {
        psi.cov=cbind(psi.cov,rep(data$unitcov[,i],3))
        psi.covnames=c(psi.covnames,unitcovnames[i])
      }
    }
  names(psi.cov)=psi.covnames
  psi_mat<-model.matrix(psi,psi.cov)

  len<-N*data$nsurveys ## length of covariates in detection design matrix
  SP<-as.factor(c(rep("A",len), rep("B",len), rep("A",len), rep("B",2*len))) # pA,pB,rA,rBA,rBa
  contrasts(SP)<-contr.treatment(2,contrasts=sp.contr)

  if (!is.null(p.cov))
    if (nrow(p.cov)==data$nunits) p.cov=apply(p.cov,2,rep,data$nsurveys*5)
  if (is.matrix(p.cov)) p.cov=data.frame(p.cov)
  p.cov$SP=SP
  seasn=time=NULL; for (i in 1:data$nseasons) seasn=c(seasn,rep(i,N*data$nsurveyseason[i]))
  for (i in 1:data$nsurveys) time=c(time,rep(i,N))
  p.cov$SEASON<-as.factor(rep(seasn,5));
  p.cov$SURVEY=as.factor(rep(time,5))
  p.cov$INT_o<-as.factor(c(rep('1',2*len),rep('2',3*len)))  #  interaction: r != p
  p.cov$INT_d<-as.factor(c(rep(1,4*len),rep(2,len)))        #  interaction: rBa != rBA
  p.cov$ID<-as.factor(rep(c('pA','pB','rA','rBA','rBa'),each=len))
  pform=as.character(p); p.covnames=names(p.cov);
  if (data$nunitcov>0)
    for (i in 1:data$nunitcov) {
      if (length(grep(unitcovnames[i],pform))>0) {
        p.cov[[length(p.cov)+1]]=rep(data$unitcov[,i],5*data$nsurveys)
        p.covnames=c(p.covnames,unitcovnames[i])
      }
    }
  if (data$nsurvcov>0)
    for (i in 1:data$nsurvcov) {
      if (length(grep(names(data$survcov)[i],pform))>0) {
        p.cov[[length(p.cov)+1]]=rep(data$survcov[,i],5)
        p.covnames=c(p.covnames,names(data$survcov)[i])
      }
    }
  names(p.cov)=p.covnames
  p.cov=data.frame(p.cov)

  p_mat<-model.matrix(p,p.cov)

  #   gamma and epsilon built-in covariates
  len=N*(data$nseasons-1)
  if (multiseason) {
    SP<-as.factor(c(rep("A",2*len),rep("B",4*len)))
    contrasts(SP)<-contr.treatment(2,contrasts=sp.contr)
    if (is.null(gamma.cov)) gamma.cov=data.frame(SP) else gamma.cov$SP=SP
    gamma.cov$INT_Ba<-as.factor(c(rep(0,4*len),rep(1,2*len)))                  #  interaction: BA? != Ba?
    gamma.cov$INT_B_a<-as.factor(c(rep(0,3*len),rep(1,len),rep(0,len),rep(1,len)))  #  interaction: B?A != B?a
    gamma.cov$INT_Baa<-as.factor(c(rep(0,5*len),rep(1,len)))  #                             interaction: B?? != Baa
    gamma.cov$INT_Ab<-as.factor(c(rep(0,len),rep(1,len),rep(0,4*len)))  #     interaction: B?? != Baa
    SEASON=rep(1:(data$nseasons-1),each=N)
    gamma.cov$SEASON=as.factor(rep(SEASON,6))  #  Replicate for the 6 parameters
    gamma.cov$ID=as.factor(rep(c('AB','Ab','BAA','BAa','BaA','Baa'),each=len))
    if(is.null(epsilon.cov)) epsilon.cov=gamma.cov
    gamma_mat<-model.matrix(gamma,gamma.cov)
    epsilon_mat<-model.matrix(epsilon,epsilon.cov)
  }
  psi.label<-c("psiA","psiBA","psiBa"); p.label<-c("pA","pB","rA","rBA","rBa")

  gamma.label=paste0("gam",c("AB","Ab","BAA","BAa","BaA","Baa"))
  epsilon.label=paste0("eps",c("AB","Ab","BAA","BAa","BaA","Baa"))

  len<-N*data$nsurveys ## length of covariates in detection design matrix
  if (is.null(fixed)) fixed=data.frame()
  if(parameterization=="xnu"){
    ## set terms to zero if using the nu parameterisation and fix nu/rho to 1
    ii=grepl("INT",colnames(psi_mat),fixed=TRUE)
    c_index<-(1:ncol(psi_mat))[!ii]
    psi_mat[2*N+1:N,c_index]<-0
    if (sum(ii)==0)
      if (sum(fixed$parm=="nu")==0) fixed=rbind(fixed,data.frame(parameterization="nu",value=1))

    ii=grepl("INT_d",colnames(psi_mat),fixed=TRUE)
    c_index<-(1:ncol(p_mat))[!ii]
    p_mat[4*len+1:len,c_index]<-0
    if (sum(ii)==0)
      if (sum(grep("rho",fixed$param))==0)
        fixed=rbind(fixed,data.frame(param=paste0("rho[",1:data$nsurveys,']'),value=rep(1,data$nsurveys)))
  }
    ## create labels
  if (parameterization=='phi') {
    psi.label<-c("psiA","psiB","phi");    p.label<-c("pA","pB","rA","rB","delta")
  }
  if (parameterization=='nu') {
    psi.label<-c("psiA","psiBa","nu");    p.label<-c("pA","pB","rA","rBa","rho")
  }
  if (parameterization=='alfa') {
    psi.label<-c("psi-alfA","psi-alfB","psi-alfAB"); p.label<-c("pA","pB","r-alfaA","r-alfaB","r-alfaAB")
    gamma.label<-c('betaA','betaB','betaAB','gamA','gamB','gamAB')
    epsilon.label<-c('dltaA','dltaB','dltaAB','xiA','xiB','xiAB')
  }
  ## create and output temporary pao file
  ### bind each block of occupancy covariates to unitcov
  temp.cov<-cbind(psi_mat[1:N,],
                  psi_mat[N+1:N,],
                  psi_mat[2*N+1:N,])
  v1=paste0(rep(colnames(psi_mat),3),"_",rep(psi.label,each=ncol(psi_mat)))

  if (multiseason) {
    k=0
    for (i in 0:5)
      for (j in 1:(data$nseasons-1)) {
           temp.cov=cbind(temp.cov,gamma_mat[k*N+1:N,]); k=k+1
           v1=c(v1,paste0(colnames(gamma_mat),"_",gamma.label[i+1],"_",j))
      }
    k=0
    for (i in 0:5)
      for (j in 1:(data$nseasons-1)) {
        temp.cov=cbind(temp.cov,epsilon_mat[k*N+1:N,]); k=k+1
        v1=c(v1,paste0(colnames(epsilon_mat),"_",epsilon.label[i+1],"_",j))
      }
  }
  colnames(temp.cov)<-v1
  temp.pao<-data
  temp.pao$nunitcov<-ncol(temp.cov)
  temp.pao$unitcov<-as.data.frame(temp.cov)  #  temp.pao$unitcov<-as.data.frame(rbind(temp.cov,temp.cov))
  rownames(temp.pao$unitcov)<-data$unitnames

  ## bind each block of detection covariates
  temp.cov=NULL; len=N*data$nsurveys; for (i in 0:4) temp.cov=cbind(temp.cov,p_mat[i*len+1:len,])
  colnames(temp.cov)<-paste0(rep(colnames(p_mat),5),"_",rep(p.label,each=ncol(p_mat)))

  temp.pao$nsurvcov<-ncol(temp.cov)
  temp.pao$survcov=as.data.frame(temp.cov)   # temp.pao$survcov<-as.data.frame(rbind(temp.cov,temp.cov))
  temp.pao$paoname<-ifelse(is.null(data$paoname),"paodata.pao",data$paoname)
  rownames(temp.pao$survcov)<-1:dim(temp.pao$survcov)[1]

  colnames(temp.pao$unitcov)<-gsub(".Intercept.","Int",colnames(temp.pao$unitcov))
  #colnames(temp.pao$unitcov)<-paste0("psi.",colnames(temp.pao$unitcov))
  colnames(temp.pao$survcov)<-gsub(".Intercept.","Int",colnames(temp.pao$survcov))
  #colnames(temp.pao$survcov)<-paste0("p.",colnames(temp.pao$survcov))

  ## create design matrices file
  v=colnames(temp.pao$unitcov); i=grep("_psi|_nu|_phi",v)
  psi.dm<-matrix(v[i],3,ncol(psi_mat),byrow=TRUE)
  psi.dm.col<-ncol(psi.dm); rownames(psi.dm)<-psi.label; colnames(psi.dm)<-paste0("a",1:ncol(psi_mat))

  temp.names<-unlist(lapply(1:5,function(xx){
    rep(colnames(temp.pao$survcov)[((xx-1)*ncol(p_mat)+1):(xx*ncol(p_mat))],temp.pao$nsurveys)
  }))

  p.dm<-matrix(temp.names,nrow=5*temp.pao$nsurveys,ncol=ncol(p_mat),byrow=TRUE)
  p.dm.col<-ncol(p.dm)
  rownames(p.dm)<-paste0(rep(p.label,each=temp.pao$nsurveys),"[",1:temp.pao$nsurveys,"]")
  colnames(p.dm)<-paste0(ifelse(is.null(gamma),"b","d"),1:ncol(p_mat))

  gamma.dm.col=epsilon.dm.col=0; gamma.dm=epsilon.dm=NULL
  if (multiseason) {
    for (j in 1:length(gamma.label)) {
      i=grep(gamma.label[j],v);
      if (length(i)>0) gamma.dm=rbind(gamma.dm,matrix(v[i],data$nseasons-1,ncol(gamma_mat),byrow=T))
    }
    gamma.dm.col<-ncol(gamma.dm);
    rownames(gamma.dm)  <-paste0(rep(gamma.label,  each=temp.pao$nseasons-1),"[",1:(temp.pao$nseasons-1),"]")
    colnames(gamma.dm)<-paste0("b",1:ncol(gamma_mat));

    for (j in 1:length(epsilon.label)) {
      i=grep(epsilon.label[j],v);
      if (length(i)>0) epsilon.dm<-rbind(epsilon.dm,matrix(v[i],data$nseasons-1,ncol(epsilon_mat),byrow=T))
    }
    epsilon.dm.col<-ncol(epsilon.dm);
    rownames(epsilon.dm)<-paste0(rep(epsilon.label,each=temp.pao$nseasons-1),"[",1:(temp.pao$nseasons-1),"]")
    colnames(epsilon.dm)<-paste0("c",1:ncol(epsilon_mat))

    if(!is.null(fixed)){
      if (multiseason) fixed$idx<-match(fixed$param,unlist(lapply(list(psi.dm,gamma.dm,epsilon.dm,p.dm),rownames))) else
                       fixed$idx<-match(fixed$param,unlist(lapply(list(psi.dm,p.dm),rownames)))
    }

    psi.dm=simplify_dm(psi.dm,temp.pao);  p.dm=simplify_dm(p.dm,temp.pao)
    if (multiseason) {
      gamma.dm=simplify_dm(gamma.dm,temp.pao);
      epsilon.dm=simplify_dm(epsilon.dm,temp.pao)
    }
    npar<-psi.dm.col+p.dm.col+gamma.dm.col+epsilon.dm.col
    if (parameterization=="alfa") colnames(psi.dm)=gsub('psi','alfa',colnames(psi.dm))
    if(!is.null(fixed)){
      if (multiseason) fixed$idx<-match(fixed$param,unlist(lapply(list(psi.dm,gamma.dm,epsilon.dm,p.dm),rownames))) else
                       fixed$idx<-match(fixed$param,unlist(lapply(list(psi.dm,p.dm),rownames)))
    }
    temp.pao=delete_unused_covs(list(psi.dm,gamma.dm,epsilon.dm,p.dm),temp.pao)
    if (outfile=='savepao') writePao(temp.pao)

    rv<-runPresence(temp.pao,list(psi.dm,gamma.dm,epsilon.dm,p.dm,NULL,NULL),
                    model,modname=modname,fixed=fixed,initvals=initvals,outfile=outfile,miscopts=miscopts)
  } else {             #  not multi-season
    npar<-psi.dm.col+p.dm.col
    if(!is.null(fixed)){
      fixed$idx<-match(fixed$param,unlist(lapply(list(psi.dm,p.dm),rownames)))
    }
    psi.dm=simplify_dm(psi.dm,temp.pao);  p.dm=simplify_dm(p.dm,temp.pao)
    rv<-runPresence(temp.pao,list(psi.dm,p.dm,NULL,NULL,NULL,NULL),model,modname=modname,
                         fixed=fixed,initvals=initvals,outfile=outfile,miscopts=miscopts)
  }

  ### extract results from output file
   v=readLines(outfile)

   ### get PRESENCE version
   i=grep("Version",v,fixed=TRUE)[1]; version=gsub(".+Version ","",v[i])

   ## find AIC value for each model
   i=grep("-2log.likelihood",v); neg2loglike=as.numeric(gsub(".+= ","",v[i])); aic=as.numeric(gsub(".+= ","",v[i+1]))

   ### get Untransformed coefficients (Beta's)
   i=grep("^A.+ : ",v); psi.coeff=matrix(as.numeric(unlist(strsplit(gsub(" +",",",gsub(".+: +","",v[i])),","))),ncol=2,byrow=T)
   rownames(psi.coeff)=gsub(" +","_",gsub(" +:.+","",v[i]))
   #i=grep("^A.+psi.+ : ",v); psi.coeff=as.numeric(gsub(" +\\S+$","",gsub(".+: +","",v[i])))
   j=grep("^B.+ : ",v); gam.coeff=matrix(as.numeric(unlist(strsplit(gsub(" +",",",gsub(".+: +","",v[j])),","))),ncol=2,byrow=T)
   rownames(gam.coeff)=gsub(" +","_",gsub(" +:.+","",v[j]))
   colnames(psi.coeff)=colnames(gam.coeff)=c('est','se')
   if (!multiseason) p.coeff=gam.coeff else {
     k=grep("^C.+ : ",v); eps.coeff=matrix(as.numeric(unlist(strsplit(gsub(" +",",",gsub(".+: +","",v[k])),","))),ncol=2,byrow=T)
     colnames(eps.coeff)=c('est','se')
     l=grep("^D.+ : ",v); p.coeff=matrix(as.numeric(unlist(strsplit(gsub(" +",",",gsub(".+: +","",v[l])),","))),ncol=2,byrow=T)
   }
   ii=grep("^[A-F].+ : ",v); names=gsub(" .+","",v[ii]); npar=length(names)

    ### get_VC matrix
  instns<-grep("Variance-Covariance Matrix of",v,fixed=TRUE)
  VC<-get_VC(v=v,offset=instns+1,n.coeff=npar,cov.names=names)

  psi.VC=VC[1:length(i), 1:length(i)]; p.VC=VC[length(i)+1:length(j),length(i)+1:length(j)]

  gam.VC=eps.VC=NULL
  if (multiseason) {
    gam.VC<-p.VC; eps.VC<-VC[length(c(i,j))+1:length(k),length(c(i,j))+1:length(k)]
    p.VC=VC[length(c(i,j,k))+1:length(l),length(c(i,j,k))+1:length(l)]
  }

  ## Get real parameter estimates from output
  rnames=temp.pao$unitnames[1:N]
  ynames=unique(gsub('\\[.+','',rownames(psi.dm)))
  y.est=list()
  for (i in 1:length(ynames))
    y.est[[i]]=get_real(real=paste0('<',ynames[i]),v=v,con.offset=1,n1=1,nunits=N,row.names=temp.pao$unitnames,fixed=fixed)
  if (length(y.est)>0) names(y.est)=ynames

  rnames=paste0(rep(temp.pao$unitnames[1:N],temp.pao$nsurveys),"_",rep(1:(temp.pao$nsurveys),each=N))
  pnames=unique(gsub('\\[.+','',rownames(p.dm)))
  p.est=list()
  for (i in 1:length(pnames))
    p.est[[i]]=get_real(real=paste0('<',pnames[i]),v=v,con.offset=1,n1=data$nsurveys,nunits=N,row.names=rnames,fixed=fixed)
  if (length(p.est)>0) names(p.est)=pnames

  rnames=paste0(rep(temp.pao$unitnames[1:N],temp.pao$nseasons-1),"_",rep(1:(temp.pao$nseasons-1),each=N))
  gnames=unique(gsub('\\[.+','',rownames(gamma.dm)))
  g.est=list()
  if (length(gnames)>0) {
    for (i in 1:length(gnames))
      g.est[[i]]=get_real(real=paste0('<',gnames[i]),v=v,con.offset=1,n1=data$nseasons-1,nunits=N,row.names=rnames,fixed=fixed)
    if (length(g.est)>0) names(g.est)=gnames
  }

  enames=unique(gsub('\\[.+','',rownames(epsilon.dm)))
  e.est=list()
  if (length(enames)>0) {
    for (i in 1:length(enames))
      e.est[[i]]=get_real(real=paste0('<',enames[i]),v=v,con.offset=1,n1=data$nseasons-1,nunits=N,row.names=rnames,fixed=fixed)
    if (length(e.est)>0) names(e.est)=gsub('gam','eps',gnames)
  }
  #  Get derived parameters...
  i=grep("Derived",v); derived=list()
  if (length(i)>0) {

    for (ii in 1:(length(i)-1)) {

      nlines=i[ii+1]-i[ii]-4; j=i[ii]
      l1=unlist(strsplit(gsub(',$','',gsub('^,','',gsub('[ -]+',',',gsub('.+:','',v[j+1])))),','))
      l2=unlist(strsplit(gsub(',$','',gsub('^,','',gsub('[ -]+',',',gsub('.+:','',v[j+2+1:nlines])))),','))
      z=grep('[0-9][0-9][0-9][0-9]',l2)
      if (length(z)>0)
        for (j in 1:length(z)) {
          zz=z[j]; l2b=as.character(l2[zz]); l2c=c(substr(l2b,1,6),substr(l2b,7,12))
          l2=c(l2[1:(zz-1)],as.numeric(l2c),l2[(zz+1):length(l2)])
          z=z+1
        }
      l3=matrix(as.numeric(l2),ncol=length(l1)*2,byrow=T)

      derived[[ii]]=list()
      for (j in 1:length(l1)) derived[[ii]][[j]] = data.frame(est=l3[,(j*2)-1],se=l3[,j*2])
      names(derived[[ii]])=l1
    }
    i=i[-length(i)]; names(derived)=v[i]

  }
  ##### check for warnings
  warn.conv<-check_conv_warn(v); warn.VC<-check_VC_warn(v)
  if (multiseason) {
   result<-list(modname=modname,
                model=list(psi=psi,p=p,gamma=gamma,epsilon=epsilon),
                dmat=list(psi=psi.dm,p=p.dm,gamma.dm=gamma.dm,epsilon.dm=epsilon.dm),
                data=temp.pao,outfile=outfile,
                neg2loglike=neg2loglike,npar=npar, aic=aic,
                beta=list(psi=as.data.frame(psi.coeff),psi.VC=psi.VC,p=as.data.frame(p.coeff),p.VC=p.VC,
                          gamma=as.data.frame(gam.coeff), gamma.VC=gam.VC, epsilon=as.data.frame(eps.coeff), epsilon.VC=eps.VC, VC=VC),
                real=list(psi=y.est, p=p.est, gamma=g.est, epsilon=e.est),
                derived=derived,
                warnings=list(conv=warn.conv,VC=warn.VC))
  } else result<-list(modname=modname,
                     model=list(psi=psi,p=p),
                     dmat=list(psi=psi.dm,p=p.dm),
                     data=temp.pao,outfile=outfile,
                     neg2loglike=neg2loglike,npar=npar, aic=aic,
                     beta=list(psi=as.data.frame(psi.coeff),psi.VC=psi.VC,p=as.data.frame(p.coeff),p.VC=p.VC,VC=VC),
                     real=list(psi=y.est, p=p.est),
                     derived=derived,
                     warnings=list(conv=warn.conv,VC=warn.VC))
   #result.save<<-result
  class(result)<-c("occMod","so2sp")
  #names(result$real)=c(rownames(psi.dm),rownames(p.dm)[grep('\\[1\\]',rownames(p.dm))])
   if(multiseason) class(result)<-c("occMod","do2sp1")
   return(result)
}
